/* -*- Mode: C++; tab-width: 4; indent-tabs-mode: t; c-basic-offset: 4 -*- */
/*===========================================================================
 * 
 * Project : Freyja
 * Author  : Mongoose
 * Website : http://icculus.org/freyja
 * Email   : mongoose@icculus.org
 * Object  : Cg plugin
 * License : No use without permission (c) 2000-2005 Mongoose
 * Comments: This is the backend of the modeler
 *
 *
 *           This file was generated using Mongoose's C++ 
 *           template generator script.  <mongoose@icculus.org>
 * 
 *-- History ------------------------------------------------- 
 *
 * 2005.07.23:
 * Mongoose - Created, Based on freyja prototype
 ==========================================================================*/

#include <string.h>
#include <math.h>
#include <hel/math.h>
#include <freyja/FreyjaFileReader.h>
#include <freyja/PerlinNoise.h>
#include <freyja/FreyjaImage.h>
#include <freyja/FreyjaPluginABI.h>
#include <mgtk/ResourceEvent.h>
#include <mgtk/mgtk_events.h>


#define CG_SUPPORT_EXT "GL_EXT_Cg_shader"


extern "C" {

	void freyja_cg_init(void (*func)(const char*, void*));
	void PerlinNoiseEventsAttach();
	void CgGUIAttach();
}

/* Setup the plugin to be used by the appliciation */
typedef struct cg_plugin_s
{
	ResourceAppPluginTest *plugin;
	void *callback;
	unsigned int gPerlinNoiseSeed = 257;
	unsigned int gPerlinNoiseW = 256;
	unsigned int gPerlinNoiseH = 256;
	unsigned int gPerlinNoiseClamp = 1;
	vec_t gPerlinNoiseIA = 1.0f;
	vec_t gPerlinNoiseIB = 2.0f;
	vec_t gPerlinNoiseD = 20.0f;
	vec4_t gColorPerlinAdd = {0.51, 0.51, 0.29, 1.0};
	vec4_t gColorPerlinMult = {0.61, 0.51, 0.35, 1.0};
	
} cg_plugin_t;


cg_plugin_t gCgPlugin;


/* Hookup the GUI and the implementation of the plugin */
void freyja_perlinnoise_init(void (*func)(const char*, void*))
{
	//CgPlugin = new cg_plugin_t;
	gCgPlugin.plugin = new ResourceAppPluginTest(CgAttach, CgGUIAttach);
	gCgPlugin.callback = 0x0;
}


void eCgProgram()
{
	PerlinNoise perlin;
	uint32 seed = 257, w = 256, h = 256, clamp = 1;
	vec_t iA = 1.0f, iB = 2.0f, d = 20.0f;
	byte *image;

	seed = gPerlinNoiseSeed;
	w = gPerlinNoiseW;
	h = gPerlinNoiseH;
	clamp = gPerlinNoiseClamp;
	iA = gPerlinNoiseIA;
	iB = gPerlinNoiseIB; 
	d = gPerlinNoiseD; 

	image = perlin.generateBuffer(w, h, seed);

	if (!image)
	{
		mgtk_print("ePerlinNoiseGen: Invalid image generation");
		return;
	}

	if (clamp)
		perlin.clampBufferIntensity(image, w, h, iA, iB, d);

	/* Modulate by a color and add a base half intensity */
 	FreyjaImage img;
	byte *rgb;
	uint32 i, n;
	img.loadPixmap(image, w, h, FreyjaImage::INDEXED_8);
	img.getImage(&rgb);

	if (!rgb)
	{
		mgtk_print("ePerlinNoiseGen: Invalid image after color conversion");
		return;
	}

	// hahaha it's 0600 no sleep -- can't wait to clean this prototype!
	for (i = 0, n = w * h * 3; i < n; ++i)
	{
		/* NOTE: No clamping or scaling of colors, however there is a 
		         weakened 50 / 50 add in the sense that ADD can only contrib
				 _up_to_ 50% of full intensity ( 255 ).

				The reason for this is to allow bleeding for plasma, etc.
		*/

		// Modulate and adjust intensity per pixel
		rgb[i] = (byte)(rgb[i] * gColorPerlinMult[0]) + 
				(byte)(128 * gColorPerlinAdd[0]);
		++i;
		rgb[i] = (byte)(rgb[i] * gColorPerlinMult[1]) +
				(byte)(128 * gColorPerlinAdd[1]);
		++i;
		rgb[i] = (byte)(rgb[i] * gColorPerlinMult[2]) + 
				(byte)(128 * gColorPerlinAdd[2]);
	}

#ifdef BUILT_INTO_APPLICATION
	freyja_load_texture_buffer(rgb, w, h, 24);
#else
	unsigned int tid = freyjaTextureCreateBuffer(rgb, 3, w, h, RGB_24);
	unsigned long id = Resource::mInstance->getIntByName("eTextureUpload");

	mgtk_print("!Uploading texture %i, using function %i...", tid, id);

	if (ResourceEvent::listen(id - 10000, tid))
		mgtk_print("!ePerlinNoiseGen generated texture successfully.");

	freyjaTextureDelete(tid);
#endif

	mgtk_event_gl_refresh();
}


void ePerlinNoiseSeed(unsigned int i)
{
	gPerlinNoiseSeed = i;
}


void ePerlinNoiseW(unsigned int i)
{
	gPerlinNoiseW = i;
}


void ePerlinNoiseH(unsigned int i)
{
	gPerlinNoiseH = i;
}


void ePerlinNoiseClamp(unsigned int i)
{
	gPerlinNoiseClamp = i;
}


void ePerlinNoiseIA(vec_t v)
{
	gPerlinNoiseIA = v;
}


void ePerlinNoiseIB(vec_t v)
{
	gPerlinNoiseIB = v;
}


void ePerlinNoiseD(vec_t v)
{
	gPerlinNoiseD = v;
}


void eDialogPerlinNoise()
{
	mgtk_event_dialog_visible_set(Resource::mInstance->getIntByName("eDialogPerlinNoise"), 1);
} 


void eColorPerlinMult(float *c, unsigned long count)
{
	uint32 i;

	for (i = 0; i < 4; ++i)
		gColorPerlinMult[i] = c[i];

	mgtk_event_set_color(Resource::mInstance->getIntByName("eColorPerlinMult"), c[0], c[1], c[2], c[3]);
	mgtk_event_gl_refresh();
}


void eColorPerlinAdd(float *c, unsigned long count)
{
	uint32 i;

	for (i = 0; i < 4; ++i)
		gColorPerlinAdd[i] = c[i];

	mgtk_event_set_color(Resource::mInstance->getIntByName("eColorPerlinAdd"), c[0], c[1], c[2], c[3]);
	mgtk_event_gl_refresh();
}


#include <math.h>
#include <stdio.h>
#include <stdlib.h>

#ifdef __APPLE__
#include <GLUT/glut.h>
#else
#include <GL/glut.h>
#endif
#include <Cg/cg.h>
#include <Cg/cgGL.h>

#ifndef CWD
# define CWD ""
#endif

/******************************************************************************/
/*** Static Data                                                            ***/
/******************************************************************************/



GLfloat LightDiffuse[] = {1.0, 0.0, 0.0, 1.0};  
GLfloat LightPosition[] = {1.0, 1.0, 1.0, 0.0};  

GLfloat CubeNormals[6][3] = 
 {  
  {-1.0, 0.0, 0.0}, {0.0, 1.0, 0.0}, {1.0, 0.0, 0.0},
  {0.0, -1.0, 0.0}, {0.0, 0.0, 1.0}, {0.0, 0.0, -1.0} 
 };

GLint CubeFaces[6][4] = 
 {  
  {0, 1, 2, 3}, {3, 2, 6, 7}, {7, 6, 5, 4},
  {4, 5, 1, 0}, {5, 6, 2, 1}, {7, 4, 0, 3} 
 };

GLfloat CubeVertices[8][3];  

/******************************************************************************/

#define MATRIX_INDEX(i, j) (j + i * 4)


static void CheckCgError(void)
{
  CGerror err = cgGetError();

  if (err != CG_NO_ERROR)
   {
     printf("CG error: %s\n", cgGetErrorString(err));
     exit(1);
   }
}


static void DrawCube(void)
 {
  int i;

  cgGLBindProgram(Program);
  CheckCgError();

  /*
   * Set various uniform parameters including the ModelViewProjection
   * matrix for transforming the incoming position into HPOS.
   */
  if(KdParam != NULL)
   cgGLSetParameter4f(KdParam, 1.0, 1.0, 0.0, 1.0);
    
  /* Set the concatenate modelview and projection matrices */
  if(ModelViewProjParam != NULL)
    cgGLSetStateMatrixParameter(ModelViewProjParam,
                                CG_GL_MODELVIEW_PROJECTION_MATRIX,
                                CG_GL_MATRIX_IDENTITY);

  cgGLEnableProfile(profile);


  /*
   * Create cube with per-vertex varying attributes 
   */
  for(i = 0; i < 6; i++) 
   {
    glBegin(GL_QUADS);

    glNormal3fv(&CubeNormals[i][0]);
    cgGLSetParameter3f(TestColorParam, 1.0, 0.0, 0.0);
    glVertex3fv(&CubeVertices[CubeFaces[i][0]][0]);

    cgGLSetParameter3f(TestColorParam, 0.0, 1.0, 0.0);
    glVertex3fv(&CubeVertices[CubeFaces[i][1]][0]);

    cgGLSetParameter3f(TestColorParam, 0.0, 0.0, 1.0);
    glVertex3fv(&CubeVertices[CubeFaces[i][2]][0]);

    cgGLSetParameter3f(TestColorParam, 1.0, 1.0, 1.0);
    glVertex3fv(&CubeVertices[CubeFaces[i][3]][0]);
    
    glEnd();
   }

  cgGLDisableProfile(profile);
 }

static void Display(void)
 {
  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
  DrawCube();
  glutSwapBuffers();
 }

static void InitializeCube(GLfloat v[8][3])
 {
  /* Setup cube vertex data. */
  v[0][0] = v[1][0] = v[2][0] = v[3][0] = -1;
  v[4][0] = v[5][0] = v[6][0] = v[7][0] = 1;
  v[0][1] = v[1][1] = v[4][1] = v[5][1] = -1;
  v[2][1] = v[3][1] = v[6][1] = v[7][1] = 1;
  v[0][2] = v[3][2] = v[4][2] = v[7][2] = 1;
  v[1][2] = v[2][2] = v[5][2] = v[6][2] = -1;
 }

static void InitializeGlut(int *argc, char *argv[])
 {
  glutInit(argc, argv);
  glutInitDisplayMode(GLUT_DOUBLE | GLUT_RGB | GLUT_DEPTH);
  glutCreateWindow(argv[0]);
  glutDisplayFunc(Display);

  InitializeCube(CubeVertices);

  /* Enable a single OpenGL light. */
  glLightfv(GL_LIGHT0, GL_DIFFUSE, LightDiffuse);
  glLightfv(GL_LIGHT0, GL_POSITION, LightPosition);
  glEnable(GL_LIGHT0);
#if 0
  glEnable(GL_LIGHTING);
#else
  glDisable(GL_LIGHTING);
#endif

  /* Use depth buffering for hidden surface elimination. */
  glEnable(GL_DEPTH_TEST);

  /* Setup the view of the cube. */
  glMatrixMode(GL_PROJECTION);
  gluPerspective( /* field of view in degree */ 40.0,
    /* aspect ratio */ 1.0,
    /* Z near */ 1.0, /* Z far */ 10.0);
  glMatrixMode(GL_MODELVIEW);
  gluLookAt(0.0, 0.0, 5.0,  /* eye is at (0,0,5) */
            0.0, 0.0, 0.0,  /* center is at (0,0,0) */
            0.0, 1.0, 0.);  /* up is in positive Y direction */

  /* Adjust cube position to be asthetic angle. */
  glTranslatef(0.0, 0.0, -1.0);
#if 1
  glRotatef(60, 1.0, 0.0, 0.0);
  glRotatef(-20, 0.0, 0.0, 1.0);
#endif
  
 }


int main(int argc, char *argv[])
 {
  InitializeGlut(&argc, argv);

  if (cgGLIsProfileSupported(CG_PROFILE_VP20))
    profile = CG_PROFILE_VP20;
  else if (cgGLIsProfileSupported(CG_PROFILE_ARBVP1))
    profile = CG_PROFILE_ARBVP1;
  else
   {
    fprintf(stderr, "Video card does not support vertex programs, exiting...\n");
    exit(-1);
   }

  /* Test cgContext creation */
  Context = cgCreateContext();
  CheckCgError();

  /* Test adding source text to context */
  Program = cgCreateProgramFromFile(Context,
                                    CG_SOURCE, CWD "cgGL_vertex_example.cg",
                                    profile,
                                    NULL, NULL);
  CheckCgError();

fprintf(stderr, "LAST LISTING----%s----\n", cgGetLastListing(Context));

fprintf(stderr, "---- PROGRAM BEGIN ----\n%s---- PROGRAM END ----\n",
        cgGetProgramString(Program, CG_COMPILED_PROGRAM));

  if(Program != NULL)
   {
    cgGLLoadProgram(Program);
    CheckCgError();

    KdParam = cgGetNamedParameter(Program, "Kd");
    CheckCgError();

    ModelViewProjParam = cgGetNamedParameter(Program, "ModelViewProj");
    CheckCgError();

    TestColorParam = cgGetNamedParameter(Program, "IN.TestColor");
    CheckCgError();
   }

  glutMainLoop();

  cgDestroyProgram(Program);
  cgDestroyContext(Context);

  return 0;
 }


void CgAttach()
{
	ResourceEventCallback::add("eCgProgram", &eCgProgram);

	// FIXME: Add limits and a GUI generator wrapper for this
	//        the GUI generator wrapper will have to wait until 
	//        interface is done to call itself back to generate
	// FIXME: Also find a way to make these data members of the other event
	//        if possible ( remember callbacks might need functions )
	ResourceEventCallback::add("eDialogCg", &eDialogCg);

	ResourceEventCallbackUInt::add("ePerlinNoiseSeed", &ePerlinNoiseSeed);
	ResourceEventCallbackUInt::add("ePerlinNoiseW", &ePerlinNoiseW);
	ResourceEventCallbackUInt::add("ePerlinNoiseH", &ePerlinNoiseH);
	ResourceEventCallbackUInt::add("ePerlinNoiseClamp", &ePerlinNoiseClamp);

	ResourceEventCallbackVec::add("ePerlinNoiseIA", &ePerlinNoiseIA);
	ResourceEventCallbackVec::add("ePerlinNoiseIB", &ePerlinNoiseIB);
	ResourceEventCallbackVec::add("ePerlinNoiseD", &ePerlinNoiseD);

	ResourceEventCallbackVecArray::add("eColorPerlinMult", &eColorPerlinMult);
	ResourceEventCallbackVecArray::add("eColorPerlinAdd", &eColorPerlinAdd);
}


void CgGUIAttach()
{
	char *filename;
	char *basename = "plugins/cg.mlisp";
	int id, menuId;

	id = Resource::mInstance->getIntByName("eDialogCg");
	menuId = Resource::mInstance->getIntByName("ePluginMenu");
	mgtk_append_item_to_menu(menuId, "Cg", id);

	filename = mgtk_rc_map(basename);
	Resource::mInstance->Load(filename);
	delete [] filename;

	id = Resource::mInstance->getIntByName("eColorPerlinAdd");
	mgtk_event_set_color(id, 0.51f, 0.51f, 0.29f, 1.0f);
	
	id = Resource::mInstance->getIntByName("eColorPerlinMult");
	mgtk_event_set_color(id, 0.61f, 0.51f, 0.35f, 1.0f);
}






